import * as functions from "firebase-functions";
import * as admin from "firebase-admin";

admin.initializeApp();

/**
 * Trigger na zapis noveho dokumentu do kolekce bufferRecords
 */
export const parseRecordsFromBuff = functions.region('europe-west1').firestore.document(`bufferRecords/{docId}`).onCreate(snapshot => {
  const buffRecords = snapshot.data();

  // Ulozeni jednotlivych polozek z bufferu do samostatnych dokumentu v prislusne kolekci
  if (!!buffRecords?.deviceId && buffRecords?.data?.length > 0) {
    let i;
    for (i = 0; i < buffRecords.data.length; i++) {

      if (!!buffRecords.data[i].timestamp?.seconds) {
        const timestampId = new Date(buffRecords.data[i].timestamp.seconds).getTime().toString();
        functions.logger.log(`Timestamp id: ${timestampId}`);

        buffRecords.data[i].timestamp = new Date(buffRecords.data[i].timestamp.seconds*1000);
        
        if (!!timestampId) {
          admin.firestore().collection(`${buffRecords.deviceId}`).doc(timestampId).create(buffRecords.data[i]);
        }
        else {
          functions.logger.log(`ERR_TIMESTAMP_ID: Timestamp id undefined`);
        }
      }
    }

    if (i === buffRecords.data.length) {
      functions.logger.info(`OK_PARSE_COMPLETE: Parse complete, device id: ${buffRecords.deviceId}`);
    }
    else {
      functions.logger.error(`ERR_PARSE_COMPLETE: Parse error, device id: ${buffRecords.deviceId}`);
    }

    // Smazani zpracovaneho dokumentu v kolekci bufferRecords
    admin.firestore().collection('bufferRecords').doc(snapshot.id).delete()
      .then(
        res => {
          functions.logger.log(`OK_PARSE_DELETE: Buffer data deleted successfully. Device id: ${buffRecords.deviceId}, doc id:  ${snapshot.id}`);
        },
        reject => {
          functions.logger.error(`ERR_PARSE_DELETE: Error deleting buffer data. Device id: ${buffRecords.deviceId}, doc id:  ${snapshot.id}`);
        }
      )
      .catch(err => {
        functions.logger.error(`ERR_PARSE_DELETE: Error deleting buffer data. Device id: ${buffRecords.deviceId}, doc id:  ${snapshot.id}`);
      });
  }
  else {
    const msg = "Could not parse data from buffer, data incomplete. " + !buffRecords.deviceId ? "Device ID missing" : "Measured data missing";
    functions.logger.error(`ERR_DATA_INCOMPLETE: ${msg}`);
  }
});