import { Directive, HostListener, Input } from '@angular/core';
import { AbstractControl, NG_VALIDATORS, Validator } from '@angular/forms';

/**
 * Validátor čísel pro matInput
 */
@Directive({
  selector: '[numberValidator]',
  providers: [{ provide: NG_VALIDATORS, useExisting: NumberValidatorDirective, multi: true }]
})
export class NumberValidatorDirective implements Validator {
  @Input('numMin') numMin: number = null;
  @Input('numMax') numMax: number = null;

  control: AbstractControl;

  ngOnChanges() {
    // Vynucení validace při změně povoleného rozsahu
    this.validate(this.control);
  }

  validate(control: AbstractControl): { [key: string]: any } {
    if (!this.control) {
      this.control = control;
    }

    if (!this.control?.value) {
      return null;
    }

    this.convertInputsToNum();

    let value = null;
    if (this.isNumber(this.control.value)) {
      value = Number(this.control.value);
    }
    else {
      return { 'formatInvalid': true };
    }
    
    if (!!this.numMin && value < this.numMin) {
      return { 'minError': true }; 
    }
    else if (!!this.numMax && value > this.numMax) {
      return { 'maxError': true }; 
    }
    else {
      return null;
    }
  }

  /**
   * Zjistí, zda je daná hodnota číslo
   * @param value 
   */
  isNumber(value: string | number): boolean {
    return ((value != null) && (value !== '') && !isNaN(Number(value.toString())));
  }

  /**
   * Převede minNum a maxNum na number, pokud je to možné
   */
  convertInputsToNum() {
    if (this.isNumber(this.numMin)) {
      this.numMin = Number(this.numMin);
    }

    if (this.isNumber(this.numMax)) {
      this.numMax = Number(this.numMax);
    }
  }

}