import { Component, OnInit, ViewChild } from '@angular/core';
import { NgForm } from '@angular/forms';
import { MatSelectChange } from '@angular/material/select';
import { MatSnackBar } from '@angular/material/snack-bar';
import { DeviceSettings } from '../data-model.interface';
import { EnvMonitorService } from '../services/env-monitor.service';

@Component({
  selector: 'app-nastaveni',
  templateUrl: './nastaveni.component.html',
  styleUrls: ['./nastaveni.component.scss']
})
export class NastaveniComponent implements OnInit {

  settingsList: DeviceSettings[] = [];

  selectedDevice: DeviceSettings = new DeviceSettings();

  @ViewChild('form') form: NgForm;

  constructor(
    private envMonitorService: EnvMonitorService,
    private snackBar: MatSnackBar
  ) { }

  ngOnInit() {
    this.loadData();
  }

  private loadData() {
    this.envMonitorService.getSettings().subscribe(
      (res) => {
        this.settingsList = res.map(item => {
          return {
            id: item.payload.doc.id,
            ...item.payload.doc.data() as DeviceSettings
          }
        });

        if (this.settingsList?.length > 0) {
          // Opětovné zvolení aktuální položky po uložení a přenačtení dat
          let index = this.settingsList.findIndex(item => item.id === this.selectedDevice?.id);
          if (index === -1) {
            index = 0;
          }
          Object.assign(this.selectedDevice, this.settingsList[index]);
        }
      },
      () => {
        this.snackBar.open('Při načítání dat došlo k chybě.', 'Zavřít', {panelClass: 'err-snackbar'});
      }
    );
  }

  compareWith(object1: DeviceSettings, object2: DeviceSettings) {
    return object1 && object2 && object1.id === object2.id;
  }

  selectedIdChanged(event: MatSelectChange) {
    if (!event) {
      return;
    }

    Object.assign(this.selectedDevice, event);
  }

  updateValidity() {
    // Workaround pro správnou validaci s matInput
    setTimeout(() => {
      for (let key in this.form.form.controls) {
        this.form.form.controls[key].updateValueAndValidity();
      }
    }, 20);
  }

  getMeasureTimeMax(): number {
    return (this.selectedDevice.sendTime < 3600) ? Number(this.selectedDevice.sendTime) : 3600;
  }

  save() {
    this.envMonitorService.updateDeviceSettings(this.selectedDevice,
      () => {
        this.snackBar.open('Změny byly úspěšně uloženy.', 'Zavřít', {panelClass: 'suc-snackbar', duration: 2000});
      },
      () => {
        this.snackBar.open('Během ukládání došlo k chybě.', 'Zavřít', {panelClass: 'err-snackbar'});
        this.loadData();
      });
  }
}
