import { Component, Input, OnInit } from '@angular/core';
import { MatSnackBar } from '@angular/material/snack-bar';
import { DeviceSettings, DeviceState } from 'src/app/data-model.interface';
import { EnvMonitorService } from 'src/app/services/env-monitor.service';

@Component({
  selector: 'udaje-zarizeni',
  templateUrl: './udaje-zarizeni.component.html',
  styleUrls: ['./udaje-zarizeni.component.scss']
})
export class UdajeZarizeniComponent implements OnInit {
  @Input('deviceSettingsList')
  set extDeviceSettingsList(deviceSettingsList: DeviceSettings[]) {
    if (!!deviceSettingsList) {
      this.mapDeviceStateData(deviceSettingsList);
    }
  }

  deviceStateList: DeviceState[] = [];

  constructor(
    private envMonitorService: EnvMonitorService,
    private snackBar: MatSnackBar
  ) { }

  ngOnInit(): void {
  }

  private mapDeviceStateData(deviceSettingsList: DeviceSettings[]) {
    this.envMonitorService.getBatteryState().subscribe(
      (res) => {
        this.deviceStateList = [];
        const batteryStateList = res.map(item => {
          return {
            id: item.payload.doc.id,
            ...item.payload.doc.data() as any
          }
        });

        deviceSettingsList.forEach(item => {
          const batItem = batteryStateList.find(bat => bat.id === item.id);
          this.deviceStateList.push({
            empty: batItem.empty,
            ...item as DeviceState
          });
        });

        console.log(this.deviceStateList);
      },
      () => {
        this.snackBar.open('Při načítání dat došlo k chybě.', 'Zavřít', {panelClass: 'err-snackbar'});
      }
    );
  }

  /*private loadBatteryState() {
    this.envMonitorService.getBatteryState().subscribe(
      (res) => {
        this.batteryStateList = res.map(item => {
          return {
            id: item.payload.doc.id,
            ...item.payload.doc.data() as BatteryState
          }
        });
      },
      () => {
        this.snackBar.open('Při načítání dat došlo k chybě.', 'Zavřít', {panelClass: 'err-snackbar'});
      }
    );
  }*/

}
