import { Injectable } from '@angular/core';
import { AngularFirestore } from '@angular/fire/firestore';
import { DeviceSettings, FirestoreTimestamp } from '../data-model.interface';

@Injectable({
  providedIn: 'root'
})
export class EnvMonitorService {

  constructor(
    private firestore: AngularFirestore
  ) { }

  getSettings() {
    return this.firestore.collection('settings').snapshotChanges();
  }

  updateDeviceSettings(deviceSettings: DeviceSettings, onSuccess: () => void, onError: () => void): Promise<any> {
    const tmp = Object.assign({}, deviceSettings);
    delete tmp.id;

    tmp.sendTime = this.convertToNumber(tmp.sendTime);
    tmp.measureTime = this.convertToNumber(tmp.measureTime);

    return this.firestore.collection('settings').doc(deviceSettings.id).update(Object.assign({}, tmp))
      .then(() => {
        onSuccess();
      })
      .catch(err => {
        console.log(err);
        onError();
      });
  }

  getMeasuredData(deviceId: string, limit?: number) {
    let res;
    if (limit) {
      res = this.firestore.collection(deviceId, ref => ref.limitToLast(limit ? limit : 20).orderBy('timestamp')).snapshotChanges();
    }
    else {
      res = this.firestore.collection(deviceId, ref => ref.orderBy('timestamp')).snapshotChanges();
    }

    return res;
  }

  getBatteryState() {
    return this.firestore.collection('batteryState').snapshotChanges();
  }

  /**
   * Zkonvertuje timestamp z firestore na Date
   * @param timestamp 
   * @returns 
   */
  convertTimestampToDate(timestamp: FirestoreTimestamp) {
    if (!!timestamp?.seconds) {
      return new Date(timestamp.seconds*1000);
    }
  }

  /**
   * Zjistí, zda je daná hodnota číslo
   * @param value 
   */
   isNumber(value: any): boolean {
    return ((value != null) && (value !== '') && !isNaN(Number(value.toString())));
  }

  /**
   * Převede hodnotu na number, pokud je to možné
   * @param value Hodnota k přetypování
   */
  convertToNumber(value): number {
    if (this.isNumber(value)) {
      value = Number(value);
    }
    return value;
  }
}
