#include "hdc2080.h"
#include "sensors.h"

#include "esp_log.h"
#include "driver/i2c.h"
#include "driver/gpio.h"

#define HDC2080_TEMP_L_REG                  0x0
#define HDC2080_TEMP_H_REG                  0x1
#define HDC2080_MEAS_CONF_REG               0x0F
#define HDC2080_MEAS_CONF_REG_START_MSK     0x1
#define HDC2080_DRDY_CONF_REG               0x0E
#define HDC2080_DRDY_CONF_DRDY_EN_MSK       0x4
#define HDC2080_INTERRUPT_CONF_REG          0x07
#define HDC2080_INTERRUPT_CONF_DRDY_EN_MSK  0x80
#define HDC2080_DRDY_GPIO                   21

#define TAG "HDC2080"

static RTC_DATA_ATTR float hdc2080_temp;
static RTC_DATA_ATTR float hdc2080_humid;

esp_err_t hdc2080_measure() {
  uint8_t data[4];

  gpio_reset_pin(HDC2080_DRDY_GPIO);
  gpio_set_direction(HDC2080_DRDY_GPIO, GPIO_MODE_INPUT);
  
  esp_err_t ret = sens_i2c_write_byte_to_reg(I2C_NUM_0, HDC2080_ADDR, HDC2080_DRDY_CONF_REG, HDC2080_DRDY_CONF_DRDY_EN_MSK);
  ret = sens_i2c_write_byte_to_reg(I2C_NUM_0, HDC2080_ADDR, HDC2080_INTERRUPT_CONF_REG, HDC2080_INTERRUPT_CONF_DRDY_EN_MSK);

  ret = sens_i2c_write_byte_to_reg(I2C_NUM_0, HDC2080_ADDR, HDC2080_MEAS_CONF_REG, HDC2080_MEAS_CONF_REG_START_MSK);

  if (ret == ESP_OK) {
    uint16_t timeout = 0;
    while ((gpio_get_level(HDC2080_DRDY_GPIO) != 0) && (timeout < 10)) {
      timeout++;
      vTaskDelay((1/portTICK_PERIOD_MS) + 1);
    }

    ret = sens_i2c_read_from_reg(I2C_NUM_0, HDC2080_ADDR, HDC2080_TEMP_L_REG, data, sizeof(data));

    if (ret == ESP_OK) {
      hdc2080_temp = (((data[1] << 8 | data[0])*165.0)/(65536.0))-40;
      hdc2080_humid = ((data[3] << 8 | data[2])*100.0)/65536.0;
      printf("%f\n", hdc2080_temp);
      printf("%f\n", hdc2080_humid);
    }
    else if (ret == ESP_ERR_TIMEOUT) {
      ESP_LOGE(TAG, "I2C Timeout");
    }
    else {
      ESP_LOGE(TAG, "Temp read error");
    }
  }
  else {
    ESP_LOGE(TAG, "HDC2080 init error");
  }

  return ret;
}

float hdc2080_get_temp() {
  return hdc2080_temp;
}

float hdc2080_get_humid() {
  return hdc2080_humid;
}