#include <stdio.h>
#include "driver/ledc.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

#include "rgb.h"

#define RGB_RED_GPIO        18
#define RGB_GREEN_GPIO      5
#define RGB_BLUE_GPIO       19
#define RGB_RED_CHANNEL     LEDC_CHANNEL_0
#define RGB_GREEN_CHANNEL   LEDC_CHANNEL_1
#define RGB_BLUE_CHANNEL    LEDC_CHANNEL_2
#define LEDS_CH_NUM         3
#define SPEED_MODE          LEDC_HIGH_SPEED_MODE
#define LED_TIMER_MAX       8191                    // 2^13-1

void rgb_pwm_init() {
    // Configuration of timers that will be used by LED Controller
    ledc_timer_config_t ledc_timer = {
        .duty_resolution = LEDC_TIMER_13_BIT, // resolution of PWM duty
        .freq_hz = 5000,                      // frequency of PWM signal
        .speed_mode = LEDC_HIGH_SPEED_MODE,   // timer mode
        .timer_num = LEDC_TIMER_0,            // timer index
        .clk_cfg = LEDC_AUTO_CLK,              // Auto select the source clock
    };

    ledc_timer_config(&ledc_timer);

    // Configuration for each channel of LED Controller
    uint8_t rgb_gpios[LEDS_CH_NUM] = {RGB_RED_GPIO, RGB_GREEN_GPIO, RGB_BLUE_GPIO};

    for (int i = 0; i < LEDS_CH_NUM; i++) {
        ledc_channel_config_t ledc_channel = {
            .channel    = i,
            //.duty       = 0,
            .duty       = LED_TIMER_MAX,
            .gpio_num   = rgb_gpios[i],
            .speed_mode = SPEED_MODE,
            .hpoint     = 0,
            .timer_sel  = LEDC_TIMER_0
        };

        ledc_channel_config(&ledc_channel);
        ledc_stop(SPEED_MODE, i, 1);
    }

    // Initialize fade service.
    ledc_fade_func_install(0);
}

void rgb_set_duty(uint8_t red_duty, uint8_t green_duty, uint8_t blue_duty) {
    ledc_set_duty_and_update(SPEED_MODE, RGB_RED_CHANNEL, ((100.0-red_duty)/100.0) * LED_TIMER_MAX, 0);
    ledc_set_duty_and_update(SPEED_MODE, RGB_GREEN_CHANNEL, ((100.0-green_duty)/100.0) * LED_TIMER_MAX, 0);
    ledc_set_duty_and_update(SPEED_MODE, RGB_BLUE_CHANNEL, ((100.0-blue_duty)/100.0) * LED_TIMER_MAX, 0);
}

void rgb_light_ms(uint8_t red_duty, uint8_t green_duty, uint8_t blue_duty, uint32_t time_ms) {
    rgb_set_duty(red_duty, green_duty, blue_duty);
    vTaskDelay(time_ms / portTICK_PERIOD_MS);
    rgb_stop();
}

void rgb_stop() {
    for (int i = 0; i < LEDS_CH_NUM; i++) {
        ledc_stop(SPEED_MODE, i, 1);
    }
}