#include "esp_sntp.h"
#include "sys/time.h"
#include "esp_log.h"

#include "sntp_time.h"

#define TAG "SNTP_TIME"

static void initialize_sntp() {
  ESP_LOGI(TAG, "Initializing SNTP");
  sntp_setoperatingmode(SNTP_OPMODE_POLL);
  sntp_setservername(0, "pool.ntp.org");
  sntp_init();
}

esp_err_t sntp_time_obtain_time(void) {
  esp_err_t ret = ESP_OK;

  initialize_sntp();

  // Wait for time to be set
  int retry = 0;
  const int retry_count = 10;
  sntp_sync_status_t sync_status = SNTP_SYNC_STATUS_RESET;
  while (sync_status == SNTP_SYNC_STATUS_RESET && ++retry < retry_count) {
    ESP_LOGI(TAG, "Waiting for system time to be set... (%d/%d)", retry, retry_count);
    vTaskDelay(2000 / portTICK_PERIOD_MS);
    sync_status = sntp_get_sync_status();
  }

  if (sync_status != SNTP_SYNC_STATUS_COMPLETED) {
    ret = ESP_FAIL;
  }

  return ret;
}

esp_err_t sntp_time_get_current_time() {
  esp_err_t ret = ESP_OK;

  time_t now;
  struct tm timeinfo;
  char strftime_buf[64];

  ret = sntp_time_obtain_time();
  time(&now);

  setenv("TZ", "CET-1CEST,M3.5.0,M10.5.0/3", 1);
  tzset();
  localtime_r(&now, &timeinfo);
  strftime(strftime_buf, sizeof(strftime_buf), "%c", &timeinfo);
  ESP_LOGI(TAG, "The current date/time in Prague is: %s", strftime_buf);

  return ret;
}
